(function (global, factory) {
	typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/common'), require('@angular/forms'), require('rxjs/BehaviorSubject'), require('lodash')) :
	typeof define === 'function' && define.amd ? define(['exports', '@angular/core', '@angular/common', '@angular/forms', 'rxjs/BehaviorSubject', 'lodash'], factory) :
	(factory((global['ngx-super-table'] = global['ngx-super-table'] || {}),global.ng.core,global.ng.common,global.ng.forms,global.Rx,global._));
}(this, (function (exports,_angular_core,_angular_common,_angular_forms,rxjs_BehaviorSubject,lodash) { 'use strict';

var __decorate$2 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata$1 = (undefined && undefined.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var sortCycle = ['ASC', 'DESC', null];
var getNextSortOrder = function (currentSortOrder) {
    var nextIndex = (sortCycle.indexOf(currentSortOrder) + 1) % sortCycle.length;
    return sortCycle[nextIndex];
};
var SuperTableState = (function () {
    function SuperTableState() {
        // publicly exposed properties
        this.hasAnyFilters = false;
        this.sortStack = [];
        // source of observable
        this.stateChangedSource = new rxjs_BehaviorSubject.BehaviorSubject(this);
        this.stateChanged$ = this.stateChangedSource.asObservable();
    }
    SuperTableState.prototype.setColumns = function (columns) {
        var _this = this;
        this.columns = columns.map(function (c) {
            if (!!c.filter) {
                _this.hasAnyFilters = true;
            }
            return {
                id: c.id,
                filterValue: null,
                sortOrder: null,
                isHidden: false,
                width: c.width || null,
                def: c,
                hasSort: !!c.sort,
                hasFilter: !!c.filter
            };
        });
    };
    SuperTableState.prototype.toggleSort = function (colState, doNotClear) {
        // Set next sort order
        colState.sortOrder = getNextSortOrder(colState.sortOrder);
        // Check if we are clearing the rest of the sort stack or not
        if (doNotClear) {
            var curIndex = this.sortStack.indexOf(colState);
            if (curIndex === -1) {
                this.sortStack.push(colState);
            }
            else if (!colState.sortOrder) {
                this.sortStack.splice(curIndex, 1);
            }
        }
        else {
            this.sortStack = colState.sortOrder ? [colState] : [];
            this.columns.forEach(function (column) {
                if (column !== colState) {
                    column.sortOrder = null;
                }
            });
        }
        this.notify();
    };
    SuperTableState.prototype.notify = function () {
        this.stateChangedSource.next(this);
    };
    return SuperTableState;
}());
SuperTableState = __decorate$2([
    _angular_core.Injectable(),
    __metadata$1("design:paramtypes", [])
], SuperTableState);

var __decorate$1 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (undefined && undefined.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
exports.SuperTable = (function () {
    function SuperTable(el, state) {
        this.el = el;
        this.state = state;
        // properties
        this.isReady = false;
        this.hasError = false;
    }
    SuperTable.prototype.ngOnInit = function () {
        var _this = this;
        this.subscription = this.state.stateChanged$.subscribe(function () { return _this.sortAndFilterRows(); });
    };
    SuperTable.prototype.ngAfterContentInit = function () {
        if (this.options.autoHeight) {
            var parentHeight = this.el.nativeElement.parentElement.clientHeight;
            this.setTableHeight(parentHeight);
        }
        this.isReady = true;
    };
    SuperTable.prototype.ngOnChanges = function (changes) {
        // Inform state of columns changes
        if (changes['columns'] && changes['columns'].isFirstChange()) {
            this.state.setColumns(changes['columns'].currentValue);
        }
        this.sortAndFilterRows();
    };
    SuperTable.prototype.ngOnDestroy = function () {
        this.subscription.unsubscribe();
    };
    SuperTable.prototype.setTableHeight = function (totalHeight) {
        // calculate header height
        var headerHeight = this.el.nativeElement.querySelector('super-table-head').offsetHeight;
        // subtract it from totalHeight, set bodyHeight to result
        this.bodyHeight = totalHeight - headerHeight;
    };
    SuperTable.prototype.sortAndFilterRows = function () {
        var _this = this;
        // Filtering
        var activeFilterColumns = this.state.columns.filter(function (c) {
            return !!c.def.filter && !!c.def.filter.isActive(c.filterValue);
        });
        if (activeFilterColumns.length) {
            this.filteredSortedRows = this.rows.filter(function (row) {
                for (var i = 0; i < activeFilterColumns.length; i++) {
                    var colState = activeFilterColumns[i];
                    var val = row[colState.def.key];
                    var filterResult = colState.def.filter.fn(colState.filterValue, val, row);
                    if (filterResult === false) {
                        return false;
                    }
                }
                return true;
            });
        }
        else {
            this.filteredSortedRows = this.rows.slice();
        }
        // Sorting
        this.filteredSortedRows.sort(function (a, b) {
            for (var i = 0; i < _this.state.sortStack.length; i++) {
                var colState = _this.state.sortStack[i];
                var val1 = a[colState.def.key];
                var val2 = b[colState.def.key];
                var compareResult = colState.sortOrder === 'ASC'
                    ? colState.def.sort(val1, val2, a, b)
                    : colState.def.sort(val2, val1, b, a);
                if (compareResult !== 0) {
                    return compareResult;
                }
            }
            return 0;
        });
    };
    return SuperTable;
}());
__decorate$1([
    _angular_core.Input(),
    __metadata("design:type", Array)
], exports.SuperTable.prototype, "rows", void 0);
__decorate$1([
    _angular_core.Input(),
    __metadata("design:type", Array)
], exports.SuperTable.prototype, "columns", void 0);
__decorate$1([
    _angular_core.Input(),
    __metadata("design:type", Object)
], exports.SuperTable.prototype, "options", void 0);
__decorate$1([
    _angular_core.Input(),
    __metadata("design:type", Object)
], exports.SuperTable.prototype, "tableClasses", void 0);
exports.SuperTable = __decorate$1([
    _angular_core.Component({
        selector: 'super-table',
        template: "\n    <super-table-head [tableClasses]=\"tableClasses\"></super-table-head>\n\n    <super-table-body\n      *ngIf=\"isReady\"\n      [rows]=\"filteredSortedRows\"\n      [tableClasses]=\"tableClasses\"\n      [options]=\"options\"\n      [style.height]=\"bodyHeight + 'px'\"\n      [bodyHeight]=\"bodyHeight\">\n    </super-table-body>\n\n    <div\n      class=\"loading-message\"\n      *ngIf=\"!isReady && !hasError\">\n      Loading...\n    </div>\n\n    <div *ngIf=\"hasError\">An error occurred.</div>\n  ",
        styles: ["\n    :host {\n      position: relative;\n      display: block;\n    }\n    .loading-message {\n      text-align: center;\n    }\n  "],
        providers: [SuperTableState]
    }),
    __metadata("design:paramtypes", [_angular_core.ElementRef, SuperTableState])
], exports.SuperTable);

var __decorate$3 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata$2 = (undefined && undefined.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var SuperTableHead = (function () {
    function SuperTableHead(state) {
        this.state = state;
    }
    return SuperTableHead;
}());
__decorate$3([
    _angular_core.Input(),
    __metadata$2("design:type", Object)
], SuperTableHead.prototype, "tableClasses", void 0);
SuperTableHead = __decorate$3([
    _angular_core.Component({
        selector: 'super-table-head',
        template: "\n    <table [ngClass]=\"tableClasses\">\n      <thead>\n        <tr>\n          <th scope=\"col\" *ngFor=\"let column of state.columns\" table-header [column]=\"column\" [ngClass]=\"{ hasSort: column.hasSort }\"></th>\n        </tr>\n        <tr *ngIf=\"state.hasAnyFilters\" class=\"filter-row\">\n          <td *ngFor=\"let column of state.columns\">\n            <div *ngIf=\"column.hasFilter\" [ngSwitch]=\"column.def.filter.type\">\n              <div *ngSwitchCase=\"'TEXT'\" text-filter [filter]=\"column.def.filter\" [column]=\"column\"></div>\n              <div *ngSwitchCase=\"'ENUM'\" enum-filter [filter]=\"column.def.filter\" [column]=\"column\"></div>\n            </div>\n          </td>\n        </tr>\n      </thead>\n    </table>\n  ",
        styles: ["\n    :host {\n      overflow-y: scroll;\n      display: block;\n    }\n    .hasSort {\n      cursor: pointer;\n    }\n    table {\n      table-layout: fixed;\n      width: 100%;\n      margin-bottom: 0;\n      border-bottom: none;\n    }\n    .filter-row td {\n      padding: 0;\n      vertical-align: middle;\n    }\n  "]
    }),
    __metadata$2("design:paramtypes", [SuperTableState])
], SuperTableHead);

var __decorate$4 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata$3 = (undefined && undefined.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var DEFAULT_ROW_HEIGHT = 20;
var PADDING_ROW_COUNT = 20;
var DEBOUNCE_DELAY = 250;
var SuperTableBody = (function () {
    function SuperTableBody(el, state) {
        var _this = this;
        this.el = el;
        this.state = state;
        this.visibleRows = [];
        // assume small row height at first.
        // The real height will be detected once rows are rendered.
        this.rowHeight = DEFAULT_ROW_HEIGHT;
        this.rowOffset = 0;
        this.scrollHandler = lodash.debounce(function () {
            _this.updateVisibleRows();
        }, DEBOUNCE_DELAY);
        this.resizeHandler = lodash.debounce(function () {
            _this.detectRowHeight();
            _this.updateVisibleRows();
        });
    }
    SuperTableBody.prototype.ngOnChanges = function (changes) {
        this.updateVisibleRows();
    };
    SuperTableBody.prototype.trackScroll = function () {
        this.scrollHandler();
    };
    SuperTableBody.prototype.onWindowResize = function () {
        this.resizeHandler();
    };
    SuperTableBody.prototype.updateVisibleRows = function () {
        var _this = this;
        var startIndex, endIndex;
        var currentScroll = this.el.nativeElement.scrollTop;
        startIndex = Math.floor(currentScroll / this.rowHeight - PADDING_ROW_COUNT);
        startIndex = Math.max(0, startIndex);
        this.rowOffset = startIndex;
        endIndex = Math.ceil((currentScroll + this.bodyHeight) / this.rowHeight + PADDING_ROW_COUNT);
        endIndex = Math.min(this.rows.length - 1, endIndex);
        this.visibleRows = this.rows.slice(startIndex, endIndex);
        setTimeout(function () {
            _this.detectRowHeight();
        });
    };
    SuperTableBody.prototype.detectRowHeight = function () {
        var tr = this.el.nativeElement.querySelector('tbody.visible-rows tr');
        if (tr != null) {
            this.rowHeight = tr.offsetHeight;
        }
    };
    return SuperTableBody;
}());
__decorate$4([
    _angular_core.Input(),
    __metadata$3("design:type", Array)
], SuperTableBody.prototype, "rows", void 0);
__decorate$4([
    _angular_core.Input(),
    __metadata$3("design:type", Object)
], SuperTableBody.prototype, "tableClasses", void 0);
__decorate$4([
    _angular_core.Input(),
    __metadata$3("design:type", Number)
], SuperTableBody.prototype, "bodyHeight", void 0);
__decorate$4([
    _angular_core.Input(),
    __metadata$3("design:type", Object)
], SuperTableBody.prototype, "options", void 0);
__decorate$4([
    _angular_core.HostListener('scroll'),
    __metadata$3("design:type", Function),
    __metadata$3("design:paramtypes", []),
    __metadata$3("design:returntype", void 0)
], SuperTableBody.prototype, "trackScroll", null);
__decorate$4([
    _angular_core.HostListener('resize'),
    __metadata$3("design:type", Function),
    __metadata$3("design:paramtypes", []),
    __metadata$3("design:returntype", void 0)
], SuperTableBody.prototype, "onWindowResize", null);
SuperTableBody = __decorate$4([
    _angular_core.Component({
        selector: 'super-table-body',
        template: "\n    <table [ngClass]=\"tableClasses\">\n      <thead class=\"sizing-thead\">\n        <tr>\n          <th scope=\"col\" *ngFor=\"let column of state.columns\" table-header [column]=\"column\" [noHeight]=\"true\"></th>\n        </tr>\n      </thead>\n      <tbody\n        class=\"dummy-rows\"\n        dummy-rows\n        [columnCount]=\"state.columns.length\"\n        [rowHeight]=\"rowHeight\"\n        [rowCount]=\"rowOffset\">\n      </tbody>\n      <tbody class=\"visible-rows\">\n        <tr *ngFor=\"let row of visibleRows\" super-table-row [row]=\"row\"></tr>\n      </tbody>\n      <tbody\n        class=\"dummy-rows\"\n        dummy-rows\n        [columnCount]=\"state.columns.length\"\n        [rowHeight]=\"rowHeight\"\n        [rowCount]=\"rows.length - rowOffset - visibleRows.length - 1\">\n      </tbody>\n    </table>\n  ",
        styles: ["\n    :host {\n      display: block;\n      overflow: auto;\n    }\n    table {\n      table-layout: fixed;\n      width: 100%;\n      margin-bottom: 0;\n    }\n    thead.sizing-thead th {\n      padding: 0 !important;\n      border-width: 0;\n    }\n    tbody.dummy-rows, tbody.visible-rows {\n      border-top: none;\n    }\n  "]
    }),
    __metadata$3("design:paramtypes", [_angular_core.ElementRef, SuperTableState])
], SuperTableBody);

var __decorate$5 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata$4 = (undefined && undefined.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var SuperTableRow = (function () {
    function SuperTableRow(state) {
        this.state = state;
    }
    return SuperTableRow;
}());
__decorate$5([
    _angular_core.Input(),
    __metadata$4("design:type", Object)
], SuperTableRow.prototype, "row", void 0);
SuperTableRow = __decorate$5([
    _angular_core.Component({
        selector: '[super-table-row]',
        template: "<td *ngFor=\"let column of state.columns\" table-cell [row]=\"row\" [column]=\"column\"></td>"
    }),
    __metadata$4("design:paramtypes", [SuperTableState])
], SuperTableRow);

var __decorate$6 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata$5 = (undefined && undefined.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var DummyRows = (function () {
    function DummyRows() {
        this.BG_IMAGE_DATA = [
            'data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAAAEAAAAlCAYAAACDKIOp',
            'AAAAGXRFWHRTb2Z0d2FyZQBBZG9iZSBJbWFnZVJlYWR5ccllPAAAABtJREFUeNpiuH',
            'v37n+G////MzAxAMHQIQACDAC7twbaN2nkgwAAAABJRU5ErkJggg=='
        ].join('');
    }
    DummyRows.prototype.rowStyleHeight = function () {
        var height = this.rowHeight * this.rowCount;
        return height + "px";
    };
    DummyRows.prototype.backgroundSize = function () {
        return "auto " + this.rowHeight + "px";
    };
    return DummyRows;
}());
__decorate$6([
    _angular_core.Input(),
    __metadata$5("design:type", Number)
], DummyRows.prototype, "rowHeight", void 0);
__decorate$6([
    _angular_core.Input(),
    __metadata$5("design:type", Number)
], DummyRows.prototype, "rowCount", void 0);
__decorate$6([
    _angular_core.Input(),
    __metadata$5("design:type", Number)
], DummyRows.prototype, "columnCount", void 0);
DummyRows = __decorate$6([
    _angular_core.Component({
        selector: '[dummy-rows]',
        template: "\n    <tr [style.height]=\"rowStyleHeight()\">\n      <td\n        [attr.colspan]=\"columnCount\"\n        [style.backgroundImage]=\"'url(' + BG_IMAGE_DATA + ')'\"\n        [style.backgroundSize]=\"backgroundSize()\">\n      </td>\n    </tr>\n  ",
        styles: ["\n    :host {\n      border: none !important;\n    }\n    td {\n      padding: 0 !important;\n      border: none !important;\n      background-repeat: repeat;\n      background-position: 0 -1px;\n    }\n\n  "]
    })
], DummyRows);

var __decorate$7 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata$6 = (undefined && undefined.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var Resizer = Resizer_1 = (function () {
    function Resizer(el) {
        this.el = el;
    }
    Resizer.prototype.grab = function (grabEvt) {
        var _this = this;
        grabEvt.preventDefault();
        var mousedownTime = Date.now();
        var initClientX = grabEvt.clientX;
        var initWidth = this.column.width || this.getActualParentWidth();
        var drag = function (event) {
            var change = event.clientX - initClientX;
            _this.column.width = Math.max(initWidth + change, Resizer_1.MIN_COLUMN_WIDTH);
        };
        var unbindDrag = function () {
            window.removeEventListener('mousemove', drag);
            window.removeEventListener('mouseup', unbindDrag);
            if (Date.now() - mousedownTime < Resizer_1.MAX_CLICK_WAIT) {
                _this.column.width = null;
            }
        };
        window.addEventListener('mousemove', drag);
        window.addEventListener('mouseup', unbindDrag);
    };
    Resizer.prototype.getActualParentWidth = function () {
        return this.el.nativeElement.parentElement.offsetWidth;
    };
    Resizer.prototype.stopClick = function (event) {
        event.preventDefault();
        event.stopPropagation();
    };
    return Resizer;
}());
Resizer.MAX_CLICK_WAIT = 250;
Resizer.MIN_COLUMN_WIDTH = 30;
__decorate$7([
    _angular_core.Input(),
    __metadata$6("design:type", Object)
], Resizer.prototype, "column", void 0);
__decorate$7([
    _angular_core.Input(),
    __metadata$6("design:type", Number)
], Resizer.prototype, "actualWidth", void 0);
Resizer = Resizer_1 = __decorate$7([
    _angular_core.Component({
        selector: '[resizer]',
        template: "<div class=\"notch\" [ngClass]=\"{ explicit: column.width }\"></div>",
        host: {
            '(click)': 'stopClick($event)',
            '(mousedown)': 'grab($event)',
            '[attr.title]': '"Click-and-drag to resize. Click to clear specified width."'
        },
        styles: ["\n    :host {\n      position: absolute;\n      right: 0;\n      top: 0;\n      width: 5px;\n      height: 100%;\n      cursor: col-resize;\n    }\n    .notch.explicit {\n      background-color: rgba(22, 140, 239, 0.2);\n    }\n    .notch {\n      width: 100%;\n      height: 50%;\n      transform: translateY(50%);\n      box-shadow: inset 1px 0 #DDD;\n    }\n  "]
    }),
    __metadata$6("design:paramtypes", [_angular_core.ElementRef])
], Resizer);
var TableHeader = (function () {
    function TableHeader(el, state) {
        this.el = el;
        this.state = state;
        this.noHeight = false;
        this.SORT_TITLE = 'Click to change sort order. Shift-click to sort on multiple columns.';
    }
    TableHeader.prototype.getWidth = function () {
        return (typeof this.column.width === 'number') ? this.column.width + 'px' : 'auto';
    };
    TableHeader.prototype.getValue = function () {
        return this.column.def.label;
    };
    TableHeader.prototype.handleClick = function (event) {
        event.preventDefault();
        if (this.column.hasSort) {
            this.state.toggleSort(this.column, event.shiftKey);
        }
    };
    return TableHeader;
}());
__decorate$7([
    _angular_core.Input(),
    __metadata$6("design:type", Object)
], TableHeader.prototype, "column", void 0);
__decorate$7([
    _angular_core.Input(),
    __metadata$6("design:type", Object)
], TableHeader.prototype, "noHeight", void 0);
TableHeader = __decorate$7([
    _angular_core.Component({
        selector: '[table-header]',
        host: {
            '[style.width]': 'getWidth()',
            '(click)': 'handleClick($event)'
        },
        template: "\n    <div *ngIf=\"!noHeight\" class=\"table-header-div\" title=\"SORT_TITLE\">\n      <span *ngIf=\"column.def.sort\" class=\"sort-icon\">\n        <span [ngSwitch]=\"column.sortOrder\">\n          <span class=\"asc-sort glyphicon glyphicon-sort-by-attributes\" *ngSwitchCase=\"'ASC'\"></span>\n          <span class=\"desc-sort glyphicon glyphicon-sort-by-attributes-alt\" *ngSwitchCase=\"'DESC'\"></span>\n          <span class=\"no-sort glyphicon glyphicon-sort\" *ngSwitchDefault></span>\n        </span>\n      </span>\n      {{ getValue() }}\n    </div>\n    <div *ngIf=\"!noHeight && !column.def.lockWidth\" resizer [column]=\"column\"></div>\n  ",
        styles: ["\n    :host {\n      position:relative;\n      -webkit-touch-callout: none;\n      -webkit-user-select: none;\n      -khtml-user-select: none;\n      -moz-user-select: none;\n      -ms-user-select: none;\n      user-select: none;\n    }\n    :host:hover .sort-icon {\n      opacity: 1;\n    }\n    .table-header-div {\n      position: relative;\n    }\n    .sort-icon {\n      font-size: 70%;\n      opacity: 1;\n      color: #168cef;\n      text-shadow: 0 1px 2px rgba(22, 140, 239, 0.6);\n    }\n    .sort-icon .no-sort {\n      opacity: 0.3;\n      text-shadow: none;\n      color: black;\n    }\n  "]
    }),
    __metadata$6("design:paramtypes", [_angular_core.ElementRef, SuperTableState])
], TableHeader);
var Resizer_1;

var superTableSorters = {
    STRING: function (val1, val2) {
        return val1.localeCompare(val2);
    },
    NUMBER: function (val1, val2) {
        return val1 - val2;
    }
};

// For date filter
var unitmap = {};
unitmap['second'] = unitmap['sec'] = unitmap['s'] = 1000;
unitmap['minute'] = unitmap['min'] = unitmap['m'] = unitmap['second'] * 60;
unitmap['hour'] = unitmap['hr'] = unitmap['h'] = unitmap['minute'] * 60;
unitmap['day'] = unitmap['d'] = unitmap['hour'] * 24;
unitmap['week'] = unitmap['wk'] = unitmap['w'] = unitmap['day'] * 7;
unitmap['month'] = unitmap['week'] * 4;
unitmap['year'] = unitmap['yr'] = unitmap['y'] = unitmap['day'] * 365;
var clauseExp = /(\d+(?:\.\d+)?)\s*([a-z]+)/;
function parseDateFilter(str) {
    // split on clauses (if any)
    var clauses = str.trim().split(',');
    var total = 0;
    // parse each clause
    for (var i = 0; i < clauses.length; i++) {
        var clause = clauses[i].trim();
        var terms = clauseExp.exec(clause);
        if (!terms) {
            continue;
        }
        var count = parseFloat(terms[1]);
        var unit = terms[2].replace(/s$/, '');
        if (!unitmap.hasOwnProperty(unit)) {
            continue;
        }
        total += count * unitmap[unit];
    }
    return total;
}
var superTableFilters = {
    STRING: {
        type: 'TEXT',
        title: 'Search by text, eg. "foo". Use "!" to ' +
            'exclude and "=" to match exact text, e.g. ' +
            '"!bar" or "=baz".',
        isActive: function (filterValue) {
            return !!filterValue && !!filterValue.trim();
        },
        placeholder: 'string search',
        fn: function (term, value, row) {
            term = term.toLowerCase().trim();
            value = String(value).toLowerCase();
            var first = term[0];
            // negate
            if (first === '!') {
                term = term.substr(1);
                if (term === '') {
                    return true;
                }
                return value.indexOf(term) === -1;
            }
            // strict
            if (first === '=') {
                term = term.substr(1);
                return term === value.trim();
            }
            // remove escaping backslashes
            term = term.replace('\\!', '!');
            term = term.replace('\\=', '=');
            return value.indexOf(term) !== -1;
        }
    },
    NUMBER: {
        type: 'TEXT',
        title: 'Search by number, e.g. "123". Optionally use comparator ' +
            'expressions like ">=10" or "<1000". Use "~" for approx. ' +
            'int values, eg. "~3" will match "3.2"',
        isActive: function (filterValue) {
            return !!filterValue && !!filterValue.trim();
        },
        placeholder: 'number search',
        fn: function (term, value, row) {
            var parsedValue = parseFloat(value);
            term = term.trim();
            var firstTwo = term.substr(0, 2);
            var firstChar = term[0];
            var against1 = parseFloat(term.substr(1));
            var against2 = parseFloat(term.substr(2));
            if (firstTwo === '<=') {
                return parsedValue <= against2;
            }
            if (firstTwo === '>=') {
                return parsedValue >= against2;
            }
            if (firstChar === '<') {
                return parsedValue < against1;
            }
            if (firstChar === '>') {
                return parsedValue > against1;
            }
            if (firstChar === '~') {
                return Math.round(parsedValue) === against1;
            }
            if (firstChar === '=') {
                return against1 === parsedValue;
            }
            return value.toString().indexOf(term.toString()) > -1;
        }
    },
    DATE: {
        type: 'TEXT',
        title: 'Search by date. Enter a date string (RFC2822 or ' +
            'ISO 8601 date). You can also type "today", "yesterday", ' +
            '"> 2 days ago", "< 1 day 2 hours ago", etc.',
        isActive: function (filterValue) {
            return !!filterValue && !!filterValue.trim();
        },
        placeholder: 'date search',
        fn: function (term, value) {
            // today
            // yesterday
            // 1 day ago
            // 2 days ago
            // < 1 day ago
            // < 10 minutes ago
            // < 10 min ago
            // < 10 minutes, 50 seconds ago
            // > 10 min, 30 sec ago
            // > 2 days ago
            // >= 1 day ago
            term = term.trim();
            if (!term) {
                return true;
            }
            var numValue = value.valueOf();
            var nowDate = new Date();
            var now = nowDate.valueOf();
            var first_char = term[0];
            var other_chars = (term.substr(1)).trim();
            var lowerbound, upperbound;
            if (first_char === '<') {
                lowerbound = now - parseDateFilter(other_chars);
                return numValue > lowerbound;
            }
            if (first_char === '>') {
                upperbound = now - parseDateFilter(other_chars);
                return numValue < upperbound;
            }
            if (term === 'today') {
                return new Date(numValue).toDateString() === nowDate.toDateString();
            }
            if (term === 'yesterday') {
                return new Date(numValue).toDateString() === new Date(now - unitmap['d']).toDateString();
            }
            var supposedDate = new Date(term);
            if (!isNaN(supposedDate.valueOf())) {
                return new Date(numValue).toDateString() === supposedDate.toDateString();
            }
            return false;
        }
    },
    ENUM: {
        type: 'ENUM',
        title: 'Click to filter rows by discreet values',
        placeholder: 'filters',
        isActive: function (filterValue) {
            if (!filterValue) {
                return false;
            }
            var filterValueObj = filterValue;
            return Object.keys(filterValueObj).some(function (key) {
                return !filterValueObj[key];
            });
        },
        fn: function (filterValue, value, row) {
            var filterValueObj = filterValue;
            return filterValueObj[value];
        }
    }
};

var __decorate$8 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata$7 = (undefined && undefined.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
exports.SuperTableCell = (function () {
    function SuperTableCell() {
    }
    return SuperTableCell;
}());
__decorate$8([
    _angular_core.Input(),
    __metadata$7("design:type", Object)
], exports.SuperTableCell.prototype, "row", void 0);
__decorate$8([
    _angular_core.Input(),
    __metadata$7("design:type", Object)
], exports.SuperTableCell.prototype, "column", void 0);
__decorate$8([
    _angular_core.Input(),
    __metadata$7("design:type", String)
], exports.SuperTableCell.prototype, "key", void 0);
__decorate$8([
    _angular_core.Input(),
    __metadata$7("design:type", Object)
], exports.SuperTableCell.prototype, "value", void 0);
exports.SuperTableCell = __decorate$8([
    _angular_core.Component({
        template: ""
    })
], exports.SuperTableCell);

var __decorate$9 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata$8 = (undefined && undefined.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var TableCell = (function () {
    function TableCell(viewContainer, resolver) {
        this.viewContainer = viewContainer;
        this.resolver = resolver;
    }
    TableCell.prototype.getValue = function () {
        return this.row[this.column.def.key];
    };
    TableCell.prototype.getFormattedValue = function () {
        if (this.column.def.format) {
            return this.column.def.format(this.getValue(), this.row, this.column);
        }
        return this.getValue();
    };
    TableCell.prototype.ngAfterViewInit = function () {
        if (this.column.def.component) {
            if (this.componentRef) {
                this.componentRef.destroy();
            }
            var cmpFactory = this.resolver.resolveComponentFactory(this.column.def.component);
            var ctxInjector = this.cmpContainer.injector;
            this.componentRef = this.cmpContainer.createComponent(cmpFactory, 0, ctxInjector);
            var instance = this.componentRef.instance;
            instance['row'] = this.row;
            instance['column'] = this.column;
            instance['key'] = this.column.def.key;
            instance['value'] = this.getValue();
            this.componentRef.changeDetectorRef.detectChanges();
        }
    };
    TableCell.prototype.ngOnDestroy = function () {
        if (this.componentRef) {
            this.componentRef.destroy();
        }
    };
    return TableCell;
}());
__decorate$9([
    _angular_core.Input(),
    __metadata$8("design:type", Object)
], TableCell.prototype, "row", void 0);
__decorate$9([
    _angular_core.Input(),
    __metadata$8("design:type", Object)
], TableCell.prototype, "column", void 0);
__decorate$9([
    _angular_core.ViewChild('cmpContainer', { read: _angular_core.ViewContainerRef }),
    __metadata$8("design:type", _angular_core.ViewContainerRef)
], TableCell.prototype, "cmpContainer", void 0);
TableCell = __decorate$9([
    _angular_core.Component({
        selector: '[table-cell]',
        template: "\n    <span *ngIf=\"!column.def.component\" [attr.title]=\"getFormattedValue()\">{{ getFormattedValue() }}</span>\n    <span *ngIf=\"column.def.component\" #cmpContainer></span>\n  ",
        styles: ["\n    :host {\n      white-space: nowrap;\n      text-overflow: ellipsis;\n      overflow: hidden;\n    }\n  "]
    }),
    __metadata$8("design:paramtypes", [_angular_core.ViewContainerRef, _angular_core.ComponentFactoryResolver])
], TableCell);

var __decorate$10 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata$9 = (undefined && undefined.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var TextFilter = (function () {
    function TextFilter(state) {
        this.state = state;
        this.onModelChange = lodash.debounce(function () {
            this.state.notify();
        }, 200);
    }
    TextFilter.prototype.clearFilter = function () {
        this.column.filterValue = '';
        this.state.notify();
    };
    return TextFilter;
}());
__decorate$10([
    _angular_core.Input(),
    __metadata$9("design:type", Object)
], TextFilter.prototype, "filter", void 0);
__decorate$10([
    _angular_core.Input(),
    __metadata$9("design:type", Object)
], TextFilter.prototype, "column", void 0);
TextFilter = __decorate$10([
    _angular_core.Component({
        selector: '[text-filter]',
        template: "\n    <input\n      class=\"form-control input-sm\"\n      type=\"text\"\n      [(ngModel)]=\"column.filterValue\"\n      (ngModelChange)=\"onModelChange($event)\"\n      [attr.placeholder]=\"filter.placeholder\"\n      [attr.title]=\"filter.title\"\n      [ngClass]=\"{ hasContent: !!column.filterValue }\" />\n\n    <button *ngIf=\"column.filterValue\" class=\"clear-btn\" role=\"button\" (click)=\"clearFilter($event)\">&times;</button>\n  ",
        styles: ["\n    :host {\n      position: relative;\n    }\n    input {\n      width: 100%;\n      font-size: 90%;\n      border: none;\n      border-radius: 0;\n    }\n    .hasContent {\n      background: #dff7ff;\n    }\n    .clear-btn {\n      position: absolute;\n      background: transparent;\n      color: black;\n      right: 0;\n      top: 0;\n      border: none;\n      font-size: 120%;\n    }\n  "]
    }),
    __metadata$9("design:paramtypes", [SuperTableState])
], TextFilter);

var __decorate$11 = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata$10 = (undefined && undefined.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var EnumFilterDropdown = (function () {
    function EnumFilterDropdown(state, el) {
        this.state = state;
        this.el = el;
    }
    EnumFilterDropdown.prototype.ngOnInit = function () {
        var styles = this.el.nativeElement.style;
        styles.top = this.top + 'px';
        styles.left = this.left + 'px';
        styles.width = this.width + 'px';
    };
    EnumFilterDropdown.prototype.ngOnDestroy = function () {
        // to ensure that references to parent component
        // do not prevent GC
        this.destroyMe = null;
    };
    EnumFilterDropdown.prototype.onChoiceChange = function () {
        this.state.notify();
    };
    EnumFilterDropdown.prototype.showAll = function () {
        var _this = this;
        lodash.forEach(this.column.filterValue, function (val, key) {
            _this.column.filterValue[key] = true;
        });
        this.state.notify();
        this.destroyMe();
    };
    return EnumFilterDropdown;
}());
__decorate$11([
    _angular_core.Input(),
    __metadata$10("design:type", Object)
], EnumFilterDropdown.prototype, "column", void 0);
EnumFilterDropdown = __decorate$11([
    _angular_core.Component({
        selector: 'super-table-enum-filter-dropdown',
        template: "\n    <div class=\"clear-filter\">\n      <button class=\"btn btn-secondary clear-filter-btn\" role=\"button\" (click)=\"showAll()\">show all</button>\n    </div>\n    <div *ngFor=\"let choice of column.def.filterChoices\">\n      <input type=\"checkbox\" [(ngModel)]=\"column.filterValue[choice]\" (ngModelChange)=\"onChoiceChange($event)\" />\n      {{ choice }}\n    </div>\n    <button role=\"button\" class=\"close-dropdown\" (click)=\"destroyMe()\">&times;</button>\n  ",
        styles: ["\n    :host {\n      position: absolute;\n      background: white;\n      padding: 5px 10px;\n      border: 1px solid #ddd;\n      box-shadow: 0 1px 10px -1px rgba(0,0,0,0.2);\n    }\n    .clear-filter {\n      border-bottom: 1px solid #DDD;\n      padding: 5px 0;\n    }\n    .close-dropdown {\n      position: absolute;\n      top: 5px;\n      right: 10px;\n      border: none;\n      background: transparent;\n      color: #CCC;\n      display: block;\n      width: 20px;\n      height: 20px;\n      line-height: 20px;\n    }\n    .close-dropdown:hover {\n      color: #AAA;\n    }\n  "]
    }),
    __metadata$10("design:paramtypes", [SuperTableState,
        _angular_core.ElementRef])
], EnumFilterDropdown);
var EnumFilter = (function () {
    function EnumFilter(state, el, viewContainer, resolver) {
        this.state = state;
        this.el = el;
        this.viewContainer = viewContainer;
        this.resolver = resolver;
        this.disabledChoices = new Set();
        this.disabledFilterCount = 0;
    }
    EnumFilter.prototype.ngOnInit = function () {
        var _this = this;
        // initialize filtered values to include all
        this.column.filterValue = {};
        this.column.def.filterChoices.forEach(function (choice) {
            _this.column.filterValue[choice] = true;
        });
        this.subscription = this.state.stateChanged$.subscribe(function () {
            _this.disabledFilterCount = lodash.values(_this.column.filterValue)
                .filter(function (isEnabled) { return !isEnabled; })
                .length;
        });
    };
    EnumFilter.prototype.ngOnDestroy = function () {
        this.subscription.unsubscribe();
    };
    EnumFilter.prototype.toggleVisibility = function () {
        var _this = this;
        if (this.dropdown) {
            this.dropdown.destroy();
            this.dropdown = null;
        }
        else {
            var clientRect = this.el.nativeElement.getBoundingClientRect();
            var cmpFactory = this.resolver.resolveComponentFactory(EnumFilterDropdown);
            var ctxInjector = this.viewContainer.injector;
            var cmpRef = this.viewContainer.createComponent(cmpFactory, 0, ctxInjector);
            cmpRef.instance.column = this.column;
            cmpRef.instance.top = clientRect.top + clientRect.height;
            cmpRef.instance.left = clientRect.left;
            cmpRef.instance.width = clientRect.width;
            cmpRef.instance.destroyMe = function () {
                _this.toggleVisibility();
            };
            this.dropdown = cmpRef;
            document.body.appendChild(cmpRef.location.nativeElement);
        }
    };
    return EnumFilter;
}());
__decorate$11([
    _angular_core.Input(),
    __metadata$10("design:type", Object)
], EnumFilter.prototype, "filter", void 0);
__decorate$11([
    _angular_core.Input(),
    __metadata$10("design:type", Object)
], EnumFilter.prototype, "column", void 0);
EnumFilter = __decorate$11([
    _angular_core.Component({
        selector: '[enum-filter]',
        template: "\n    <button\n      [attr.title]=\"filter.title\"\n      role=\"button\"\n      (click)=\"toggleVisibility($event)\"\n      [ngClass]=\"{ hasDisabled : disabledFilterCount }\">\n      <strong>{{ filter.placeholder }}:</strong>\n      <span [hidden]=\"disabledFilterCount != 0\">showing all</span>\n      <span [hidden]=\"disabledFilterCount == 0\">filtering {{disabledFilterCount}} value(s)</span>\n    </button>\n\n  ",
        styles: ["\n    :host {\n      position: relative;\n      display: block;\n      height: 100%;\n      font-size: 90%;\n      white-space: nowrap;\n      text-overflow: ellipsis;\n    }\n    button {\n      text-align: left;\n      margin: 0 5px;\n      display: block;\n      width: calc(100% - 10px);\n      height: 100%;\n      background: #ddd;\n      border: none;\n      border-radius: 2px;\n    }\n    button.hasDisabled {\n      background: rgba(22, 140, 239, 0.2);\n    }\n    .enum-filter-dropdown {\n      position: absolute;\n      top: 100%;\n      z-index: 1;\n    }\n  "]
    }),
    __metadata$10("design:paramtypes", [SuperTableState,
        _angular_core.ElementRef,
        _angular_core.ViewContainerRef,
        _angular_core.ComponentFactoryResolver])
], EnumFilter);

var __decorate = (undefined && undefined.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
exports.SuperTableModule = (function () {
    function SuperTableModule() {
    }
    return SuperTableModule;
}());
exports.SuperTableModule = __decorate([
    _angular_core.NgModule({
        // Components declared in this library
        declarations: [
            exports.SuperTable,
            SuperTableHead,
            SuperTableBody,
            exports.SuperTableCell,
            SuperTableRow,
            DummyRows,
            TableHeader,
            TableCell,
            Resizer,
            TextFilter,
            EnumFilter,
            EnumFilterDropdown
        ],
        imports: [_angular_common.CommonModule, _angular_forms.FormsModule],
        exports: [exports.SuperTable, exports.SuperTableCell],
        entryComponents: [
            EnumFilterDropdown
        ]
    })
], exports.SuperTableModule);

exports.superTableSorters = superTableSorters;
exports.superTableFilters = superTableFilters;

Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=index.umd.js.map
