var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
import { Component, ElementRef, HostListener, Input } from '@angular/core';
import { SuperTableState } from './SuperTableState';
import { debounce } from 'lodash';
var DEFAULT_ROW_HEIGHT = 20;
var PADDING_ROW_COUNT = 20;
var DEBOUNCE_DELAY = 250;
var SuperTableBody = (function () {
    function SuperTableBody(el, state) {
        var _this = this;
        this.el = el;
        this.state = state;
        this.visibleRows = [];
        // assume small row height at first.
        // The real height will be detected once rows are rendered.
        this.rowHeight = DEFAULT_ROW_HEIGHT;
        this.rowOffset = 0;
        this.scrollHandler = debounce(function () {
            _this.updateVisibleRows();
        }, DEBOUNCE_DELAY);
        this.resizeHandler = debounce(function () {
            _this.detectRowHeight();
            _this.updateVisibleRows();
        });
    }
    SuperTableBody.prototype.ngOnChanges = function (changes) {
        this.updateVisibleRows();
    };
    SuperTableBody.prototype.trackScroll = function () {
        this.scrollHandler();
    };
    SuperTableBody.prototype.onWindowResize = function () {
        this.resizeHandler();
    };
    SuperTableBody.prototype.updateVisibleRows = function () {
        var _this = this;
        var startIndex, endIndex;
        var currentScroll = this.el.nativeElement.scrollTop;
        startIndex = Math.floor(currentScroll / this.rowHeight - PADDING_ROW_COUNT);
        startIndex = Math.max(0, startIndex);
        this.rowOffset = startIndex;
        endIndex = Math.ceil((currentScroll + this.bodyHeight) / this.rowHeight + PADDING_ROW_COUNT);
        endIndex = Math.min(this.rows.length - 1, endIndex);
        this.visibleRows = this.rows.slice(startIndex, endIndex);
        setTimeout(function () {
            _this.detectRowHeight();
        });
    };
    SuperTableBody.prototype.detectRowHeight = function () {
        var tr = this.el.nativeElement.querySelector('tbody.visible-rows tr');
        if (tr != null) {
            this.rowHeight = tr.offsetHeight;
        }
    };
    return SuperTableBody;
}());
__decorate([
    Input(),
    __metadata("design:type", Array)
], SuperTableBody.prototype, "rows", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], SuperTableBody.prototype, "tableClasses", void 0);
__decorate([
    Input(),
    __metadata("design:type", Number)
], SuperTableBody.prototype, "bodyHeight", void 0);
__decorate([
    Input(),
    __metadata("design:type", Object)
], SuperTableBody.prototype, "options", void 0);
__decorate([
    HostListener('scroll'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], SuperTableBody.prototype, "trackScroll", null);
__decorate([
    HostListener('resize'),
    __metadata("design:type", Function),
    __metadata("design:paramtypes", []),
    __metadata("design:returntype", void 0)
], SuperTableBody.prototype, "onWindowResize", null);
SuperTableBody = __decorate([
    Component({
        selector: 'super-table-body',
        template: "\n    <table [ngClass]=\"tableClasses\">\n      <thead class=\"sizing-thead\">\n        <tr>\n          <th scope=\"col\" *ngFor=\"let column of state.columns\" table-header [column]=\"column\" [noHeight]=\"true\"></th>\n        </tr>\n      </thead>\n      <tbody\n        class=\"dummy-rows\"\n        dummy-rows\n        [columnCount]=\"state.columns.length\"\n        [rowHeight]=\"rowHeight\"\n        [rowCount]=\"rowOffset\">\n      </tbody>\n      <tbody class=\"visible-rows\">\n        <tr *ngFor=\"let row of visibleRows\" super-table-row [row]=\"row\"></tr>\n      </tbody>\n      <tbody\n        class=\"dummy-rows\"\n        dummy-rows\n        [columnCount]=\"state.columns.length\"\n        [rowHeight]=\"rowHeight\"\n        [rowCount]=\"rows.length - rowOffset - visibleRows.length - 1\">\n      </tbody>\n    </table>\n  ",
        styles: ["\n    :host {\n      display: block;\n      overflow: auto;\n    }\n    table {\n      table-layout: fixed;\n      width: 100%;\n      margin-bottom: 0;\n    }\n    thead.sizing-thead th {\n      padding: 0 !important;\n      border-width: 0;\n    }\n    tbody.dummy-rows, tbody.visible-rows {\n      border-top: none;\n    }\n  "]
    }),
    __metadata("design:paramtypes", [ElementRef, SuperTableState])
], SuperTableBody);
export { SuperTableBody };
